/* $NetBSD: example.h,v 1.2 2011/12/04 23:55:36 jym Exp $ */

/* 
 * This file is placed in the public domain.
 */

#ifndef _SECMODEL_EXAMPLE_EXAMPLE_H_
#define	_SECMODEL_EXAMPLE_EXAMPLE_H_

#define SECMODEL_EXAMPLE_ID   "id.unique.secmodel.example"
#define SECMODEL_EXAMPLE_NAME "Example security model"

#endif /* !_SECMODEL_EXAMPLE_EXAMPLE_H_ */
