%A S. Deering
%A R. Hinden
%B RFC1883
%T Internet Protocol, Version 6 (IPv6) Specification
%D December 1995
%U ftp://ftp.isi.edu/in-notes/rfc1883.txt

%A A. Conta
%A S. Deering
%B RFC1885
%T Internet Control Message Protocol (ICMPv6) for the Internet Protocol Version 6 (IPv6) Specification
%D December 1995
%U ftp://ftp.isi.edu/in-notes/rfc1885.txt

%A R. Hinden
%A S. Deering
%B RFC2373
%T IP Version 6 Addressing Architecture
%D July 1998
%U ftp://ftp.isi.edu/in-notes/rfc2373.txt

%A J. Postel
%B RFC793
%T Transmission Control Protocol
%D Sep 1, 1981
%U ftp://ftp.isi.edu/in-notes/rfc793.txt

%A J. Postel
%A J.K. Reynolds
%B RFC959
%T File Transfer Protocol
%D Oct 1, 1985
%U ftp://ftp.isi.edu/in-notes/rfc959.txt

%A A. Durand
%A B. Buclin
%B RFC2546
%T 6Bone Routing Practice
%D March 1999
%U ftp://ftp.isi.edu/in-notes/rfc2546.txt

%A S. Deering
%A R. Hinden
%B RFC2460
%T Internet Protocol, Version 6 (IPv6) Specification
%D December 1998
%U ftp://ftp.isi.edu/in-notes/rfc2460.txt

%A T. Narten
%A E. Nordmark
%A W. Simpson
%B RFC2461
%T Neighbor Discovery for IP Version 6 (IPv6)
%D December 1998
%U ftp://ftp.isi.edu/in-notes/rfc2461.txt

%A S. Thomson
%A T. Narten
%B RFC2462
%T IPv6 Stateless Address Autoconfiguration
%D December 1998
%U ftp://ftp.isi.edu/in-notes/rfc2462.txt

%A A. Conta
%A S. Deering
%B RFC2463
%T Internet Control Message Protocol (ICMPv6) for the Internet Protocol Version 6 (IPv6) Specification
%D December 1998
%U ftp://ftp.isi.edu/in-notes/rfc2463.txt

%A T. Bates
%A Y. Rekhter
%B RFC2260
%T Scalable Support for Multi-homed Multi-provider Connectivity
%D January 1998
%U ftp://ftp.isi.edu/in-notes/rfc2260.txt

%A G. Malkin
%A R. Minnear
%B RFC2080
%T RIPng for IPv6
%D January 1997
%U ftp://ftp.isi.edu/in-notes/rfc2080.txt

%A G. Malkin
%B RFC2081
%T RIPng Protocol Applicability Statement
%D January 1997
%U ftp://ftp.isi.edu/in-notes/rfc2081.txt

%A T. Bates
%A R. Chandra
%A D. Katz
%A Y. Rekhter
%B RFC2283
%T Multiprotocol Extensions for BGP-4
%D February 1998
%U ftp://ftp.isi.edu/in-notes/rfc2283.txt

%A P. Marques
%A F. Dupont
%B RFC2545
%T Use of BGP-4 Multiprotocol Extensions for IPv6 Inter-Domain Routing
%D March 1999
%U ftp://ftp.isi.edu/in-notes/rfc2545.txt

%A P. Ferguson
%A D. Senie
%B RFC2267
%T Network Ingress Filtering: Defeating Denial of Service Attacks which employ IP Source Address Spoofing
%D January 1998
%U ftp://ftp.isi.edu/in-notes/rfc2267.txt

%A Matt Crawford
%B draft-ietf-ipngwg-router-renum-09.txt 
%T Router Renumbering for IPv6
%D June 1999
%U work in progress material

%A R. Gilligan
%A E. Nordmark
%B RFC1933
%T Transition Mechanisms for IPv6 Hosts and Routers
%D April 1996
%U ftp://ftp.isi.edu/in-notes/rfc1933.txt

%A Erik Nordmark
%B draft-ietf-ngtrans-siit-06.txt 
%T Stateless IP/ICMP Translator (SIIT)
%D June 24, 1999
%U work in progress material

%Q TIS
%T TIS Gauntlet
%U http://www.tis.com/

%A Marcus Ranum
%T Firewall Toolkit (FWTK)
%U http://www.fwtk.org/
%D first released in October 1, 1993

%A John Postel
%B RFC791
%T Internet Protocol
%D September 1981
%U ftp://ftp.isi.edu/in-notes/rfc791.txt

%A Stephen Kent
%A Randall Atkinson
%B RFC2401
%T Security Architecture for the Internet Protocol
%D November 1998
%U ftp://ftp.isi.edu/in-notes/rfc2401.txt

%A Stephen Kent
%A Randall Atkinson
%B RFC2402
%T IP Authentication Header
%D November 1998
%U ftp://ftp.isi.edu/in-notes/rfc2402.txt

%A Craig Metz
%T Porting Kernel Code to Four BSDs and Linux
%D June 1999
%B 1999 USENIX annual technical conference, Freenix track
%U http://www.usenix.org/publications/library/proceedings/usenix99/metz.html
