#! /bin/sh

# $NetBSD: id3db.sh,v 1.1 2007/04/15 15:22:44 agc Exp $
#
# Copyright © 2007 Alistair Crooks.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote
#    products derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
# GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

id3info=/usr/music/.id3info
id3db=/usr/music/.id3.db

db -w -C -E B btree $id3db

awk -v database=$id3db '
/^Filename:/	{ filename = $2 }
/^Title:/	{ title = $2 }
/^Artist:/	{ artist = $2 }
/^Album:/	{ album = $2 }
/^Year:/	{ year = $2 }
/^Genre:/	{ genre = $2 " " $3 }
/^Track:/	{ track = $2 }
/^Comment:/	{ comment = $0 }
/^Directory/	{ directory = $2 }
/^$/	{
	if (artist == "")
		artist = "Unknown"
	if (year == "")
		year = "Unknown"
	if (genre == "")
		genre = "Unknown"
	s = sprintf("db -w -E B btree %s %ca%s/%s%c %c%s%c %cy%s/%s%c %c%s%c %cg%s/%s%c %c%s%c",
		database,
		39, directory, filename, 39, 39, artist, 39,
		39, directory, filename, 39, 39, year, 39,
		39, directory, filename, 39, 39, genre, 39);
	system(s);
}
' $id3info

exit 0
